@@ -68,8 +68,10 @@ static boost::uuids::uuid makeRuleID(std::string& id)
   return gen(id);
 }
 
-void parseRuleParams(boost::optional<luaruleparams_t> params, boost::uuids::uuid& uuid)
+void parseRuleParams(boost::optional<luaruleparams_t> params, boost::uuids::uuid& uuid, uint64_t& creationOrder)
 {
+  static uint64_t s_creationOrder = 0;
+
   string uuidStr;
 
   if (params) {
@@ -79,6 +81,7 @@ void parseRuleParams(boost::optional<luaruleparams_t> params, boost::uuids::uuid
   }
 
   uuid = makeRuleID(uuidStr);
+  creationOrder = s_creationOrder++;
 }
 
 typedef std::unordered_map<std::string, boost::variant<bool, int, std::string, std::vector<std::pair<int,int> > > > ruleparams_t;
@@ -101,11 +104,11 @@ static void showRules(GlobalStateHolder<vector<T> > *someRulActions, boost::opti
 
   auto rules = someRulActions->getLocal();
   if (showUUIDs) {
-    boost::format fmt("%-3d %-38s %9d %-56s %s\n");
-    g_outputBuffer += (fmt % "#" % "UUID" % "Matches" % "Rule" % "Action").str();
+    boost::format fmt("%-3d %-38s %9d %9d %-56s %s\n");
+    g_outputBuffer += (fmt % "#" % "UUID" % "Cr. Order" % "Matches" % "Rule" % "Action").str();
     for(const auto& lim : *rules) {
       string name = lim.d_rule->toString().substr(0, truncateRuleWidth);
-      g_outputBuffer += (fmt % num % boost::uuids::to_string(lim.d_id) % lim.d_rule->d_matches % name % lim.d_action->toString()).str();
+      g_outputBuffer += (fmt % num % boost::uuids::to_string(lim.d_id) % lim.d_creationOrder % lim.d_rule->d_matches % name % lim.d_action->toString()).str();
       ++num;
     }
   }
@@ -322,7 +325,7 @@ void setupLuaRules()
       sw.start();
       for(int n=0; n < times; ++n) {
         const item& i = items[n % items.size()];
-        DNSQuestion dq(&i.qname, i.qtype, i.qclass, &i.rem, &i.rem, (struct dnsheader*)&i.packet[0], i.packet.size(), i.packet.size(), false, &sw.d_start);
+        DNSQuestion dq(&i.qname, i.qtype, i.qclass, 0, &i.rem, &i.rem, (struct dnsheader*)&i.packet[0], i.packet.size(), i.packet.size(), false, &sw.d_start);
         if(rule->matches(&dq))
           matches++;
       }
@@ -373,6 +376,10 @@ void setupLuaRules()
       return std::shared_ptr<DNSRule>(new OrRule(a));
     });
 
+  g_lua.writeFunction("DSTPortRule", [](uint16_t port) {
+      return std::shared_ptr<DNSRule>(new DSTPortRule(port));
+    });
+
   g_lua.writeFunction("TCPRule", [](bool tcp) {
       return std::shared_ptr<DNSRule>(new TCPRule(tcp));
     });
@@ -413,6 +420,10 @@ void setupLuaRules()
       return std::shared_ptr<DNSRule>(new ERCodeRule(rcode));
     });
 
+  g_lua.writeFunction("EDNSOptionRule", [](uint16_t optcode) {
+      return std::shared_ptr<DNSRule>(new EDNSOptionRule(optcode));
+    });
+
   g_lua.writeFunction("showRules", [](boost::optional<ruleparams_t> vars) {
       showRules(&g_rulactions, vars);
     });
@@ -429,6 +440,10 @@ void setupLuaRules()
       return std::shared_ptr<TimedIPSetRule>(new TimedIPSetRule());
     });
 
+  g_lua.writeFunction("PoolAvailableRule", [](std::string poolname) {
+    return std::shared_ptr<DNSRule>(new PoolAvailableRule(poolname));
+  });
+
   g_lua.registerFunction<void(std::shared_ptr<TimedIPSetRule>::*)()>("clear", [](std::shared_ptr<TimedIPSetRule> tisr) {
       tisr->clear();
     });
