@@ -109,7 +109,7 @@ shared_ptr<DNSRecordContent> DNSRecordContent::unserialize(const DNSName& qname,
 
   struct dnsrecordheader drh;
   drh.d_type=htons(qtype);
-  drh.d_class=htons(1);
+  drh.d_class=htons(QClass::IN);
   drh.d_ttl=0;
   drh.d_clen=htons(serialized.size());
 
@@ -131,7 +131,7 @@ std::shared_ptr<DNSRecordContent> DNSRecordContent::mastermake(const DNSRecord &
     return std::make_shared<UnknownRecordContent>(dr, pr);
   }
 
-  return std::shared_ptr<DNSRecordContent>(i->second(dr, pr));
+  return i->second(dr, pr);
 }
 
 std::shared_ptr<DNSRecordContent> DNSRecordContent::mastermake(uint16_t qtype, uint16_t qclass,
@@ -142,21 +142,9 @@ std::shared_ptr<DNSRecordContent> DNSRecordContent::mastermake(uint16_t qtype, u
     return std::make_shared<UnknownRecordContent>(content);
   }
 
-  return std::shared_ptr<DNSRecordContent>(i->second(content));
+  return i->second(content);
 }
 
-std::unique_ptr<DNSRecordContent> DNSRecordContent::makeunique(uint16_t qtype, uint16_t qclass,
-                                               const string& content)
-{
-  zmakermap_t::const_iterator i=getZmakermap().find(make_pair(qclass, qtype));
-  if(i==getZmakermap().end()) {
-    return std::unique_ptr<DNSRecordContent>(new UnknownRecordContent(content));
-  }
-
-  return std::unique_ptr<DNSRecordContent>(i->second(content));
-}
-
-
 std::shared_ptr<DNSRecordContent> DNSRecordContent::mastermake(const DNSRecord &dr, PacketReader& pr, uint16_t oc) {
   // For opcode UPDATE and where the DNSRecord is an answer record, we don't care about content, because this is
   // not used within the prerequisite section of RFC2136, so - we can simply use unknownrecordcontent.
@@ -171,7 +159,7 @@ std::shared_ptr<DNSRecordContent> DNSRecordContent::mastermake(const DNSRecord &
     return std::make_shared<UnknownRecordContent>(dr, pr);
   }
 
-  return std::shared_ptr<DNSRecordContent>(i->second(dr, pr));
+  return i->second(dr, pr);
 }
 
 
@@ -221,12 +209,12 @@ DNSResourceRecord DNSResourceRecord::fromWire(const DNSRecord& d) {
   return rr;
 }
 
-void MOADNSParser::init(bool query, const char *packet, unsigned int len)
+void MOADNSParser::init(bool query, const std::string& packet)
 {
-  if(len < sizeof(dnsheader))
+  if (packet.size() < sizeof(dnsheader))
     throw MOADNSException("Packet shorter than minimal header");
   
-  memcpy(&d_header, packet, sizeof(dnsheader));
+  memcpy(&d_header, packet.data(), sizeof(dnsheader));
 
   if(d_header.opcode != Opcode::Query && d_header.opcode != Opcode::Notify && d_header.opcode != Opcode::Update)
     throw MOADNSException("Can't parse non-query packet with opcode="+ std::to_string(d_header.opcode));
@@ -238,15 +226,10 @@ void MOADNSParser::init(bool query, const char *packet, unsigned int len)
 
   if (query && (d_header.qdcount > 1))
     throw MOADNSException("Query with QD > 1 ("+std::to_string(d_header.qdcount)+")");
-
-  uint16_t contentlen=len-sizeof(dnsheader);
-
-  d_content.resize(contentlen);
-  copy(packet+sizeof(dnsheader), packet+len, d_content.begin());
   
   unsigned int n=0;
 
-  PacketReader pr(d_content);
+  PacketReader pr(packet);
   bool validPacket=false;
   try {
     d_qtype = d_qclass = 0; // sometimes replies come in with no question, don't present garbage then
@@ -272,7 +255,7 @@ void MOADNSParser::init(bool query, const char *packet, unsigned int len)
       else 
         dr.d_place=DNSResourceRecord::ADDITIONAL;
 
-      unsigned int recordStartPos=pr.d_pos;
+      unsigned int recordStartPos=pr.getPosition();
 
       DNSName name=pr.getName();
 
@@ -295,7 +278,7 @@ void MOADNSParser::init(bool query, const char *packet, unsigned int len)
         dr.d_content=DNSRecordContent::mastermake(dr, pr, d_header.opcode);
       }
 
-      d_answers.push_back(make_pair(dr, pr.d_pos));
+      d_answers.push_back(make_pair(dr, pr.getPosition() - sizeof(dnsheader)));
 
       /* XXX: XPF records should be allowed after TSIG as soon as the actual XPF option code has been assigned:
          if (dr.d_place == DNSResourceRecord::ADDITIONAL && seenTSIG && dr.d_type != QType::XPF)
@@ -310,18 +293,18 @@ void MOADNSParser::init(bool query, const char *packet, unsigned int len)
           throw MOADNSException("Packet ("+d_qname.toLogString()+"|#"+std::to_string(d_qtype)+") has a TSIG record in an invalid position.");
         }
         seenTSIG = true;
-        d_tsigPos = recordStartPos + sizeof(struct dnsheader);
+        d_tsigPos = recordStartPos;
       }
     }
 
 #if 0
-    if(pr.d_pos!=contentlen) {
-      throw MOADNSException("Packet ("+d_qname+"|#"+std::to_string(d_qtype)+") has trailing garbage ("+ std::to_string(pr.d_pos) + " < " + 
-                            std::to_string(contentlen) + ")");
+    if(pr.getPosition()!=packet.size()) {
+      throw MOADNSException("Packet ("+d_qname+"|#"+std::to_string(d_qtype)+") has trailing garbage ("+ std::to_string(pr.getPosition()) + " < " +
+                            std::to_string(packet.size()) + ")");
     }
 #endif
   }
-  catch(std::out_of_range &re) {
+  catch(const std::out_of_range &re) {
     if(validPacket && d_header.tc) { // don't sweat it over truncated packets, but do adjust an, ns and arcount
       if(n < d_header.ancount) {
         d_header.ancount=n; d_header.nscount = d_header.arcount = 0;
@@ -334,7 +317,7 @@ void MOADNSParser::init(bool query, const char *packet, unsigned int len)
       }
     }
     else {
-      throw MOADNSException("Error parsing packet of "+std::to_string(len)+" bytes (rd="+
+      throw MOADNSException("Error parsing packet of "+std::to_string(packet.size())+" bytes (rd="+
                             std::to_string(d_header.rd)+
                             "), out of bounds: "+string(re.what()));
     }
@@ -383,45 +366,40 @@ void PacketReader::copyRecord(unsigned char* dest, uint16_t len)
 void PacketReader::xfr48BitInt(uint64_t& ret)
 {
   ret=0;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   ret<<=8;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   ret<<=8;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   ret<<=8;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   ret<<=8;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   ret<<=8;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
 }
 
 uint32_t PacketReader::get32BitInt()
 {
   uint32_t ret=0;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   ret<<=8;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   ret<<=8;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   ret<<=8;
-  ret+=d_content.at(d_pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   
   return ret;
 }
 
 
 uint16_t PacketReader::get16BitInt()
-{
-  return get16BitInt(d_content, d_pos);
-}
-
-uint16_t PacketReader::get16BitInt(const vector<unsigned char>&content, uint16_t& pos)
 {
   uint16_t ret=0;
-  ret+=content.at(pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   ret<<=8;
-  ret+=content.at(pos++);
+  ret+=static_cast<uint8_t>(d_content.at(d_pos++));
   
   return ret;
 }
@@ -435,10 +413,8 @@ DNSName PacketReader::getName()
 {
   unsigned int consumed;
   try {
-    DNSName dn((const char*) d_content.data() - 12, d_content.size() + 12, d_pos + sizeof(dnsheader), true /* uncompress */, 0 /* qtype */, 0 /* qclass */, &consumed, sizeof(dnsheader));
+    DNSName dn((const char*) d_content.data(), d_content.size(), d_pos, true /* uncompress */, 0 /* qtype */, 0 /* qclass */, &consumed, sizeof(dnsheader));
     
-    // the -12 fakery is because we don't have the header in 'd_content', but we do need to get 
-    // the internal offsets to work
     d_pos+=consumed;
     return dn;
   }
@@ -457,7 +433,7 @@ static string txtEscape(const string &name)
   char ebuf[5];
 
   for(string::const_iterator i=name.begin();i!=name.end();++i) {
-    if((unsigned char) *i > 127 || (unsigned char) *i < 32) {
+    if((unsigned char) *i >= 127 || (unsigned char) *i < 32) {
       snprintf(ebuf, sizeof(ebuf), "\\%03u", (unsigned char)*i);
       ret += ebuf;
     }
@@ -482,7 +458,7 @@ string PacketReader::getText(bool multi, bool lenField)
     }
     uint16_t labellen;
     if(lenField)
-      labellen=d_content.at(d_pos++);
+      labellen=static_cast<uint8_t>(d_content.at(d_pos++));
     else
       labellen=d_recordlen - (d_pos - d_startrecordpos);
     
@@ -504,7 +480,7 @@ string PacketReader::getUnquotedText(bool lenField)
 {
   uint16_t stop_at;
   if(lenField)
-    stop_at = (uint8_t)d_content.at(d_pos) + d_pos + 1;
+    stop_at = static_cast<uint8_t>(d_content.at(d_pos)) + d_pos + 1;
   else
     stop_at = d_recordlen;
 
@@ -835,7 +811,7 @@ uint32_t getDNSPacketLength(const char* packet, size_t length)
   }
   try
   {
-    const dnsheader* dh = (const dnsheader*) packet;
+    const dnsheader* dh = reinterpret_cast<const dnsheader*>(packet);
     DNSPacketMangler dpm(const_cast<char*>(packet), length);
 
     const uint16_t qdcount = ntohs(dh->qdcount);
@@ -945,3 +921,49 @@ uint16_t getRecordsOfTypeCount(const char* packet, size_t length, uint8_t sectio
   }
   return result;
 }
+
+bool getEDNSUDPPayloadSizeAndZ(const char* packet, size_t length, uint16_t* payloadSize, uint16_t* z)
+{
+  if (length < sizeof(dnsheader)) {
+    return false;
+  }
+
+  *payloadSize = 0;
+  *z = 0;
+
+  try
+  {
+    const dnsheader* dh = (const dnsheader*) packet;
+    DNSPacketMangler dpm(const_cast<char*>(packet), length);
+
+    const uint16_t qdcount = ntohs(dh->qdcount);
+    for(size_t n = 0; n < qdcount; ++n) {
+      dpm.skipLabel();
+      /* type and class */
+      dpm.skipBytes(4);
+    }
+    const size_t numrecords = ntohs(dh->ancount) + ntohs(dh->nscount) + ntohs(dh->arcount);
+    for(size_t n = 0; n < numrecords; ++n) {
+      dpm.skipLabel();
+      const uint16_t dnstype = dpm.get16BitInt();
+      const uint16_t dnsclass = dpm.get16BitInt();
+
+      if(dnstype == QType::OPT) {
+        /* skip extended rcode and version */
+        dpm.skipBytes(2);
+        *z = dpm.get16BitInt();
+        *payloadSize = dnsclass;
+        return true;
+      }
+
+      /* TTL */
+      dpm.skipBytes(4);
+      dpm.skipRData();
+    }
+  }
+  catch(...)
+  {
+  }
+
+  return false;
+}
