@@ -94,15 +94,26 @@ void DNSPacketWriter::startRecord(const DNSName& name, uint16_t qtype, uint32_t
   d_sor=d_content.size(); // this will remind us where to stuff the record size
 }
 
-void DNSPacketWriter::addOpt(uint16_t udpsize, int extRCode, int Z, const vector<pair<uint16_t,string> >& options)
+void DNSPacketWriter::addOpt(const uint16_t udpsize, const uint16_t extRCode, const uint16_t ednsFlags, const optvect_t& options, const uint8_t version)
 {
   uint32_t ttl=0;
 
   EDNS0Record stuff;
 
-  stuff.extRCode=extRCode;
-  stuff.version=0;
-  stuff.Z=htons(Z);
+  stuff.version = version;
+  stuff.extFlags = htons(ednsFlags);
+
+  /* RFC 6891 section 4 on the Extended RCode wire format
+   *    EXTENDED-RCODE
+   *        Forms the upper 8 bits of extended 12-bit RCODE (together with the
+   *        4 bits defined in [RFC1035].  Note that EXTENDED-RCODE value 0
+   *        indicates that an unextended RCODE is in use (values 0 through 15).
+   */
+  // XXX Should be check for extRCode > 1<<12 ?
+  stuff.extRCode = extRCode>>4;
+  if (extRCode != 0) { // As this trumps the existing RCODE
+    getHeader()->rcode = extRCode;
+  }
 
   static_assert(sizeof(EDNS0Record) == sizeof(ttl), "sizeof(EDNS0Record) must match sizeof(ttl)");
   memcpy(&ttl, &stuff, sizeof(stuff));
@@ -110,10 +121,10 @@ void DNSPacketWriter::addOpt(uint16_t udpsize, int extRCode, int Z, const vector
   ttl=ntohl(ttl); // will be reversed later on
 
   startRecord(g_rootdnsname, QType::OPT, ttl, udpsize, DNSResourceRecord::ADDITIONAL, false);
-  for(optvect_t::const_iterator iter = options.begin(); iter != options.end(); ++iter) {
-    xfr16BitInt(iter->first);
-    xfr16BitInt(iter->second.length());
-    xfrBlob(iter->second);
+  for(auto const &option : options) {
+    xfr16BitInt(option.first);
+    xfr16BitInt(option.second.length());
+    xfrBlob(option.second);
   }
 }
 
@@ -131,7 +142,7 @@ void DNSPacketWriter::xfr48BitInt(uint64_t val)
 
 void DNSPacketWriter::xfr32BitInt(uint32_t val)
 {
-  int rval=htonl(val);
+  uint32_t rval=htonl(val);
   uint8_t* ptr=reinterpret_cast<uint8_t*>(&rval);
   d_content.insert(d_content.end(), ptr, ptr+4);
 }
