@@ -33,7 +33,7 @@
 bool g_verbose{true};
 bool g_syslog{true};
 
-BOOST_AUTO_TEST_SUITE(dnscrypt_cc)
+BOOST_AUTO_TEST_SUITE(test_dnscrypt_cc)
 
 #ifdef HAVE_DNSCRYPT
 
